@@ -25,11 +25,9 @@ from buildbot import config
 from buildbot.interfaces import IWorker
 from buildbot.process import metrics
 from buildbot.process.properties import Properties
-from buildbot.status.worker import WorkerStatus
 from buildbot.util import Notifier
 from buildbot.util import bytes2unicode
 from buildbot.util import service
-from buildbot.util.eventual import eventually
 
 
 @implementer(IWorker)
@@ -100,7 +98,7 @@ class AbstractWorker(service.BuildbotService):
         self.manager = None
         self.workerid = None
 
-        self.worker_status = WorkerStatus(name)
+        self.info = Properties()
         self.worker_commands = None
         self.workerforbuilders = {}
         self.max_builds = max_builds
@@ -139,10 +137,10 @@ class AbstractWorker(service.BuildbotService):
         self.conn = None
 
         # during disconnection self.conn will be set to None before all disconnection notifications
-        # are delivered. During that period _pending_disconnection_delivery_notifier will be set to
+        # are delivered. During that period _pending_conn_shutdown_notifier will be set to
         # a notifier and allows interested users to wait until all disconnection notifications are
         # delivered.
-        self._pending_disconnection_delivery_notifier = None
+        self._pending_conn_shutdown_notifier = None
 
         self._old_builder_list = None
         self._configured_builderid_list = None
@@ -224,21 +222,20 @@ class AbstractWorker(service.BuildbotService):
         if not info:
             return
 
-        self.worker_status.setAdmin(info.get("admin"))
-        self.worker_status.setHost(info.get("host"))
-        self.worker_status.setAccessURI(info.get("access_uri", None))
-        self.worker_status.setVersion(info.get("version", "(unknown)"))
+        # set defaults
+        self.info.setProperty("version", "(unknown)", "Worker")
 
         # store everything as Properties
         for k, v in info.items():
             if k in ('environ', 'worker_commands'):
                 continue
-            self.worker_status.info.setProperty(k, v, "Worker")
+            self.info.setProperty(k, v, "Worker")
 
     @defer.inlineCallbacks
     def _getWorkerInfo(self):
         worker = yield self.master.data.get(
             ('workers', self.workerid))
+        self._paused = worker["paused"]
         self._applyWorkerInfo(worker['workerinfo'])
 
     def setServiceParent(self, parent):
@@ -434,15 +431,14 @@ class AbstractWorker(service.BuildbotService):
         self.conn = conn
         self._old_builder_list = None  # clear builder list before proceed
 
-        self.worker_status.setConnected(True)
-
         self._applyWorkerInfo(conn.info)
         self.worker_commands = conn.info.get("worker_commands", {})
         self.worker_environ = conn.info.get("environ", {})
         self.worker_basedir = conn.info.get("basedir", None)
         self.worker_system = conn.info.get("system", None)
 
-        self.conn.notifyOnDisconnect(self.detached)
+        # The _detach_sub member is only ever used from tests.
+        self._detached_sub = self.conn.notifyOnDisconnect(self.detached)
 
         workerinfo = {
             'admin': conn.info.get('admin'),
@@ -473,7 +469,6 @@ class AbstractWorker(service.BuildbotService):
     def messageReceivedFromWorker(self):
         now = time.time()
         self.lastMessageReceived = now
-        self.worker_status.setLastMessageReceived(now)
 
     def setupProperties(self, props):
         for name in self.properties.properties:
@@ -485,27 +480,24 @@ class AbstractWorker(service.BuildbotService):
                     name, self.defaultProperties.getProperty(name), "Worker")
 
     @defer.inlineCallbacks
-    def _handle_disconnection_delivery_notifier(self):
-        self._pending_disconnection_delivery_notifier = Notifier()
-        yield self.conn.waitForNotifyDisconnectedDelivered()
-        self._pending_disconnection_delivery_notifier.notify(None)
-        self._pending_disconnection_delivery_notifier = None
+    def _handle_conn_shutdown_notifier(self, conn):
+        self._pending_conn_shutdown_notifier = Notifier()
+        yield conn.waitShutdown()
+        self._pending_conn_shutdown_notifier.notify(None)
+        self._pending_conn_shutdown_notifier = None
 
     @defer.inlineCallbacks
     def detached(self):
-        # protect against race conditions in conn disconnect path and someone
-        # calling detached directly. At the moment the null worker does that.
-        if self.conn is None:
-            return
+        conn = self.conn
+        self.conn = None
+        self._handle_conn_shutdown_notifier(conn)
+
+        # Note that _pending_conn_shutdown_notifier will not be fired until detached()
+        # is complete.
 
         metrics.MetricCountEvent.log("AbstractWorker.attached_workers", -1)
 
-        self._handle_disconnection_delivery_notifier()
-
-        yield self.conn.waitShutdown()
-        self.conn = None
         self._old_builder_list = []
-        self.worker_status.setConnected(False)
         log.msg("Worker.detached({})".format(self.name))
         self.releaseLocks()
         yield self.master.data.updates.workerDisconnected(
@@ -542,15 +534,10 @@ class AbstractWorker(service.BuildbotService):
     @defer.inlineCallbacks
     def _waitForCompleteShutdownImpl(self, conn):
         if conn:
-            d = defer.Deferred()
-
-            def _disconnected():
-                eventually(d.callback, None)
-            conn.notifyOnDisconnect(_disconnected)
-            yield d
-            yield conn.waitForNotifyDisconnectedDelivered()
-        elif self._pending_disconnection_delivery_notifier is not None:
-            yield self._pending_disconnection_delivery_notifier.wait()
+            yield conn.wait_shutdown_started()
+            yield conn.waitShutdown()
+        elif self._pending_conn_shutdown_notifier is not None:
+            yield self._pending_conn_shutdown_notifier.wait()
 
     @defer.inlineCallbacks
     def _disconnect(self, conn):
@@ -633,9 +620,9 @@ class AbstractWorker(service.BuildbotService):
         this worker will not start.
         """
 
-        # If we're waiting to shutdown gracefully or paused, then we shouldn't
+        # If we're waiting to shutdown gracefully, paused or quarantined then we shouldn't
         # accept any new jobs.
-        if self._graceful or self._paused:
+        if self._graceful or self._paused or self.quarantine_timer:
             return False
 
         if self.max_builds:
@@ -681,6 +668,7 @@ class AbstractWorker(service.BuildbotService):
     def unpause(self):
         """Restart running new builds on the worker."""
         self._paused = False
+        self.stopQuarantineTimer()
         self.botmaster.maybeStartBuildsForWorker(self.name)
         self.updateState()
 
@@ -694,7 +682,6 @@ class AbstractWorker(service.BuildbotService):
         if self.quarantine_timer:  # already in quarantine
             return
 
-        self.pause()
         self.quarantine_timer = self.master.reactor.callLater(
             self.quarantine_timeout, self.exitQuarantine)
         log.msg("{} has been put in quarantine for {}s".format(
@@ -706,14 +693,14 @@ class AbstractWorker(service.BuildbotService):
             self.quarantine_timeout = self.quarantine_max_timeout
 
     def exitQuarantine(self):
+        log.msg("{} has left quarantine".format(self.name))
         self.quarantine_timer = None
-        self.unpause()
+        self.botmaster.maybeStartBuildsForWorker(self.name)
 
     def stopQuarantineTimer(self):
         if self.quarantine_timer is not None:
             self.quarantine_timer.cancel()
-            self.quarantine_timer = None
-            self.unpause()
+            self.exitQuarantine()
 
 
 class Worker(AbstractWorker):
